#!/usr/bin/tclsh
## factorup.tcl
## by dudliohara@yahoo.com  2003.01.31  version 1.0

# changes since 0.9:
#   interactive prompting + factorup.cfg
#   loads whole delfile first, then submits

package require http 2.3  ;# ie. TCL 8.3

set default_delfile ""
set default_batchsize 10
set default_username ""
set default_cookie ""

if {$tcl_platform(platform) == "unix"} {set default_delfile "SoB.del" }
if {$tcl_platform(platform) == "windows"} {set default_delfile "sobstatus.dat"}

if {![catch {set fp [open factorup.cfg]}]} {

    while { ![eof $fp]} {

        gets $fp oneline

        switch [lindex $oneline 0] {

            username { set default_username [lindex $oneline 1]}
            cookie   { set default_cookie [lindex $oneline 1]}
            delfile  { set default_delfile [lindex $oneline 1]}
            batchsize { set default_batchsize [lindex $oneline 1]}

            default {}
        }
    }

    close $fp
}

proc  _default {string} {
    if { $string == "" } { return "" }
    return " \[$string\]"
}

set username  ""
set password  ""
set mycookie  ""
set delfile   ""
set batchsize ""


while { $username == "" } {
    puts -nonewline "Enter username[_default $default_username]: "
    flush stdout
    gets stdin username
    if {$username == ""} {set username $default_username } else { set default_cookie "" }
}

while { $mycookie == "" && $password == "" } {
    puts -nonewline "Enter password: "
    flush stdout
    gets stdin password
    if {$password == "" } { set mycookie $default_cookie }
}

while { $delfile == ""} {
    puts -nonewline "Enter filename[_default $default_delfile]: "
    flush stdout
    gets stdin delfile
    if {$delfile == ""} { set delfile $default_delfile }
}

while { $batchsize == ""} {
    puts -nonewline "Enter batchsize[_default $default_batchsize]: "
    flush stdout
    gets stdin batchsize
    if {$batchsize == ""} {
        set batchsize $default_batchsize
    } else {
        # test for integer-ness
        regexp {[0-9]+} $batchsize batchmatch
        if {![info exists batchmatch] || $batchmatch != $batchsize} {
            set batchsize ""
        } else {
            if [catch {expr $batchsize > 0}] {
                # ok, test for integer overflow
                set batchsize ""
            } else {
                if [expr $batchsize == 0] {
                    # ok, test for zero
                    set batchsize ""
                }
            }
        }
    }
}





# called every $batchsize
proc _submit { factors cookie l_verified_new} {

    set token [http::geturl http://www.seventeenorbust.com/sieve/sievePost.mhtml -query [http::formatQuery factors $factors] -headers [list Cookie $cookie]]

    upvar #0 $token state

    regexp {<h2>Verification Results</h2>(.*)<h2>} $state(body) match results

    regexp {<tt>([0-9]+) of [0-9]+} $results match num_verified

    regexp {<br>([0-9]+) of the results were new} $results match num_new

    return [list \
            [expr $num_verified + [lindex $l_verified_new 0]] \
            [expr $num_new + [lindex $l_verified_new 1]]]

}

# get cookie
http::config -useragent "Mozilla"

if {$password != ""} {

    puts "Logging in... $username"
    if {[catch {set token [http::geturl http://www.seventeenorbust.com/login/?[http::formatQuery -login true -username $username -password $password]]} catch_error]} { puts "Unable to connect"; exit }

    upvar #0 $token state

    array set meta $state(meta)

    set mycookie ""
    catch {set mycookie [lindex [split $meta(Set-Cookie) ;] 0]}
}

# test the cookie against account page
puts -nonewline "Testing login..."
flush stdout
set token [http::geturl http://www.seventeenorbust.com/account/ -headers [list Cookie $mycookie]]

upvar #0 $token state

if [regexp {Account Preferences} $state(body)] { puts "OK" } else { puts "Login failed"; exit }


# look for factors
puts "Opening factor file... $delfile"
if [catch {set fp [open $delfile r]} catch_error] { puts "Open failed"; exit }

set l_factors [list]

set num_lines -1  ;# eof counts as a line otherwise
set num_factors 0
set num_factors_local 0
set num_errors 0

set total_verified 0
set total_new 0

set l_verified_new [list 0 0]

while { ![eof $fp]} {

    incr num_lines

    gets $fp oneline

    # strip factors out of the $delfile
    if [regexp {[0-9]+ \| [0-9]+\*2\^[0-9]+\+1} $oneline match] {
        set l_factors [linsert $l_factors end $match]
    }
}

close $fp

puts "$num_lines lines"
puts "[llength $l_factors] factors"


# write config
puts "Config is good, writing... factorup.cfg"
set cfp [open "factorup.cfg" w]
puts $cfp "username $username"
puts $cfp "cookie $mycookie"
puts $cfp "delfile $delfile"
puts $cfp "batchsize $batchsize"
close $cfp


puts -nonewline "Submitting"
flush stdout


set batch ""
for {set i 1} {[expr $i <= [llength $l_factors]]} {incr i} {

    set batch $batch\n[lindex $l_factors [expr $i - 1]]

    # submit a batch every $i or at the end
    if {([expr $i % $batchsize] == 0) || ([expr $i == [llength $l_factors]])} {

        if {[catch {set l_verified_new [_submit $batch $mycookie $l_verified_new ]} ]} {
            puts -nonewline "E"
            incr num_errors
        } else {
            puts -nonewline "."
        }

        flush stdout
        set batch ""
    }
}


# how was it...
puts "\n[lindex $l_verified_new 0] Verified.  [lindex $l_verified_new 1] New."

if {[expr $num_errors > 0]} {
    puts "There were $num_errors batch errors."
    puts "  Up to [expr $num_errors * $batchsize] factors are unaccounted for.  Try running the script again."
}

puts "END"

exit

# END


